# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
#######################
#
#    This program generates a line graph of numeric data. Its use should be paired with
#    the program offset_data_reformat.py, to put the data in the correct format.
#    The data should be in a table, with the first row being the list of line IDs. The
#    first column (after the first row) will be the x axis labels. Each other column will
#    contain the data for a line on the graph. 
#
#	python offset_preformatted_cmd.py [options]
#	Options:
#	--in_dir	Input directory
#	--in_file	Input file
#	--out_dir	Output folder
#	--x_min		Minimum X value to output on graph
#	--x_max		Maximum X value to output on graph
#
##################


### Import modules
import matplotlib.pyplot as plt
from pylab import *
import sys
import numpy as np
import math
import csv
import datetime
import os
from matplotlib import rcParams
import matplotlib.ticker as ticker
from operator import add
import matplotlib.font_manager
import argparse

parser = argparse.ArgumentParser(description='Reformat the data from offset chart workflow')
parser.add_argument('--in_dir',   type=str, help='Input directory')
parser.add_argument('--in_file',  type=str, help='Input file')
parser.add_argument('--out_dir',  type=str, help='Output folder')
parser.add_argument('--x_min',  type=int, help='Minimum X value to output on graph')
parser.add_argument('--x_max',  type=int, help='Maximum X value to output on graph')


def get_args():
    '''
    Return the argparse arguments as a dictionary
    '''
    args = vars(parser.parse_args())
    return args


def main():

    # Get the command line arguments
    args = get_args()

    # Make the input and graph settings dictionaries
    input_settings = configure_input_settings(args)
    graph_settings = configure_graph_settings()

    # Open the main graph data file and line labels
    line_data_text = import_data_from_file(input_settings['input_dir']+input_settings['input_file'])

    print 'input_settings', input_settings


    # Set X range for data
    if input_settings['x_min'] is not None and input_settings['x_max'] is not None:
        #print "knarfle"
        line_data_text = data_range(input_settings, line_data_text)

    # Process the main graph data file and line labels
    line_data = parse_data_from_lines(input_settings,line_data_text)


    #print 'line_data', line_data
    # Create output directory (if necessary) and prepare output file
    if input_settings['output_dir'] is not None and input_settings['output_dir'] is not '/':
        output_folder = create_output_folder(input_settings['output_dir'])

    # Write the data to an output file
    #write_to_output(input_settings,graph_settings,line_data)

	#print "this is the line data", line_data

    # Make the line graph
    make_line_graph(graph_settings,line_data)

    # Save the line graphs to a file
    save_to_file(input_settings,graph_settings)

    # Display the graph, if selected
    if graph_settings['display_graph'] is True:
        plt.show()

    # Clear the image buffer before exiting
    plt.cla()
    plt.clf()
    plt.close()

    # Exit
    sys.exit(0)

def cm2inch(cm):
    return (cm/2.54)

def data_range(input_settings, line_data):

    # Trims the leading and trailing sets of zeros from the start and end of the
    # data. This function is really only relevent to the way the data is exported
    # by our galaxy function, and prevents extra lines from appearing in the
    # line graph.

    new_data = []
    index = 0
    is_min = True
    is_max = False
    if input_settings['x_min'] is not None:
        x_min = input_settings['x_min']
        is_min = False
        print "using x_min"
    if input_settings['x_max'] is not None:
        x_max = input_settings['x_max']
        is_max = False
        print "using x_max"
    for line in line_data:
        split_line = line.strip('\n').strip('\r').split('\t')
        print 'split_line', split_line
        if index is 0:
            new_data.append(line)
        else:
            print 'split_line[0]: ', split_line[0]
            print 'str(x_min)', str(x_min)
            print 'str(x_max)', str(x_max)
            if int(split_line[0]) == int(x_min):
                is_min = True
                print 'is_min = True'
            if int(split_line[0]) > int(x_max):
                is_max = True
                print 'is_max = True'
            if is_min is True and is_max is False:
                new_data.append(line)
                #print 'split_line', split_line
        index +=1
            #print "index: ", index
    print 'new data: ', new_data
    return new_data


def parse_data_from_lines(input_settings,datafile):

    # This functions parses the raw data into a format that can be easily
    # used, based on the columns indicated by the command line inputs. It
    # returns a list of lists

    parsed_data = dict()
    line_index = 0 #The index of the data file
    split_index = 0 #The index of the split line
    x_labels = []
    line_data = []
    for line in datafile:
        split_line = line.strip('\n').strip('\r').split('\t')
        #print "split line: ",line_index, "_",split_line
        if line_index is 0:
            line_labels = split_line
            #print line labels
            for j in range(len(line_labels)):
                #print "j: ", len(line_labels)
                line_data.append([])
            #print "line data: ", line_data
        else:
            for i in split_line:
                #print "i: ", i
                #print "split_index", split_index
                if split_index is 0:
                    x_labels.append(i)
                else:
                    if i is '.':
                        line_data[split_index-1].append(0)
                    else:
                        line_data[split_index-1].append(i)
                split_index += 1
            split_index = 0
        line_index += 1
    #print "x_labels: ", x_labels
    #print "line_labels: ", line_labels
    #print "line_data: ", line_data
    parsed_data['x_labels'] = x_labels
    parsed_data['line_labels'] = line_labels
    parsed_data['line_data'] = line_data
    print parsed_data
    return parsed_data


def create_output_folder(new_folder):

    # Checks to see if a directory exists, and if not, makes the directory. It
    # returns the directory path as output

    if not os.path.exists(new_folder):
        os.mkdir(new_folder)
    return new_folder


def configure_input_settings(args):

    # Initializes the file input and matplotlib settings. The graph settings
    # can be changed, but the input settings should not be. The function returns
    # the input and graph settings.

    input_settings = {
        'input_dir':args['in_dir'],
        'input_file':args['in_file'],
        'output_dir':args['out_dir'],
        'x_min':args['x_min'],
        'x_max':args['x_max']
        }

    if input_settings['output_dir'] is None:
        input_settings['output_dir'] = input_settings['input_dir']

    input_settings['output_file'] = input_settings['input_file']+'_OffsetOutput'
    input_settings['output_image'] = input_settings['input_file']+'_Offset'

    return input_settings

def configure_graph_settings():

    ########### These can be changed as needed to customize the graphs
    graph_settings = {
        'show_title':False,
        'show_xlabel':False,
        'show_ylabel':False,
        'show_legend':False,
        'show_legend_frame':False,
        'x_label':'Offset',
        'y_label':'Proportion',
        'alpha_labels':True,
        'label_font':{'family':'sans serif','color':'black','weight':'normal','size':16},
        'title_font':{'family':'sans serif','color':'black','weight':'normal','size':10},
        'tick_font':{'family':'sans serif','color':'black','weight':'normal','size':8},
        'legend_font_size':8,
        'tick_label_color':'black',
        'show_x_tick_labels':True,
        'show_y_tick_labels':True,
        'x_tick_label_factor':1,
        'y_tick_label_factor':(100),
        'export_svg':True,
        'export_png':False,
        'x_major_tick_locator':5,
        'x_minor_tick_locator':1,
        'y_major_tick_locator':False,
        'y_minor_tick_locator':False,
        'x_MaxNLocator':False,
        'y_MaxNLocator':2,
        'plot_height':4,
        'plot_width':8,
        'show_grid':False,
        'subplot_adjustments':{'bottom':0.1,'top':0.8,'left':0.05,'right':0.98,'wspace':0.1},
        'display_graph':False,
        'auto_colors':False,
        'colormap':'gist_ncar',
        'line_colors':{0:'blueviolet',1:'black',2:'silver',3:'skyblue',4:'green',5:'firebrick',6:'magenta',7:'cadetblue',8:'darksalmon',9:'dodgerblue',10:'darkkhaki',11:'R'},
        'line_width':1,
        'set_autoscale':True,
        'ymin_val':False,
        'ymax_val':False,
        'xmin_val':-10,
        'xmax_val':10,
        'bottom_spine':True,
        'top_spine':False,
        'left_spine':True,
        'right_spine':False,
        'headers':['Offset']
        }
    #############
    return graph_settings


def import_data_from_file(filename):

    # This function opens a file from the disk. It returns the raw data.
    try:
        return open(filename)
    except IOError:
        print "Unable to open file: %s" %(filename)


def make_line_graph(graph_settings,data_dict):

    # Makes the line graph, using the data and the graph settings

    # Create the figure and axis
    fig,ax = plt.subplots()
    #fig = plt.figure(figsize=(cm2inch(20),cm2inch(4)))
    #ax.xaxis.set_minor_locator(ticker.MultipleLocator(1))
    fig.set_size_inches(cm2inch(graph_settings['plot_width']),cm2inch(graph_settings['plot_height']))
    # Draw the lines
    for i in range(len(data_dict['line_labels'])):
        if graph_settings['auto_colors']==True:
            plt.plot(data_dict['x_labels'],data_dict['line_data'][i],
                label=str(data_dict['line_labels'][i]),
                lw=graph_settings['line_width'])
        else:
            plt.set_cmap(graph_settings['colormap'])
            print 'color: ', graph_settings['line_colors'][i]
            plt.plot(data_dict['x_labels'],data_dict['line_data'][i],
                label=str(data_dict['line_labels'][i]),
                lw=graph_settings['line_width'],
                color=graph_settings['line_colors'][i])
        #plt.show()

    # Set ticks
    if graph_settings['x_major_tick_locator'] is not False:
        ax.xaxis.set_major_locator(ticker.MultipleLocator(graph_settings['x_major_tick_locator']))
    if graph_settings['x_minor_tick_locator'] is not False:
        ax.xaxis.set_minor_locator(ticker.MultipleLocator(graph_settings['x_minor_tick_locator']))
    if graph_settings['y_major_tick_locator'] is not False:
        ax.yaxis.set_major_locator(ticker.MultipleLocator(graph_settings['y_major_tick_locator']))
    if graph_settings['y_minor_tick_locator'] is not False:
        ax.yaxis.set_minor_locator(ticker.MultipleLocator(graph_settings['y_minor_tick_locator']))
    if graph_settings['x_MaxNLocator'] is not False:
        ax.yaxis.set_major_locator(ticker.MaxNLocator(graph_settings['x_MaxNLocator']))
    if graph_settings['y_MaxNLocator'] is not False:
        ax.yaxis.set_major_locator(ticker.MaxNLocator(graph_settings['y_MaxNLocator']))
    plt.autoscale()

    # Set visibility of spines
    ax.spines['bottom'].set_visible(graph_settings['bottom_spine'])
    ax.spines['top'].set_visible(graph_settings['top_spine'])
    ax.spines['left'].set_visible(graph_settings['left_spine'])
    ax.spines['right'].set_visible(graph_settings['right_spine'])

    # Set additional tick and grid parameters
    ax.tick_params(which='both', direction='out',left='on', bottom='on', top='off',right='off')
    plt.tick_params(axis='y',which='major', direction='out',labelleft='on', labelright='off')
    # Set tick labels
    if graph_settings['show_x_tick_labels'] is True:
        ax.set_xticklabels([str(int(i*graph_settings['x_tick_label_factor'])) for i in xticks()[0]],size=8, visible=True, fontdict = graph_settings['tick_font'] )
    else:
        ax.set_xticklabels([])
    if graph_settings['show_y_tick_labels'] is True:
        ax.set_yticklabels([str(int(i*graph_settings['y_tick_label_factor'])) for i in yticks()[0]],size=8,visible=True, fontdict = graph_settings['tick_font'] )
    else:
        ax.set_yticklabels([])

    # Draw the grid
    if graph_settings['show_grid'] is True:
        plt.grid(b=True, which='major',linestyle=':')
        plt.grid(b=True, which='minor',linestyle=':')

    # Set initial margins around plot
    left_margin = 0.025
    right_margin = 0.975
    bottom_margin = 0.030
    top_margin = 0.975

    # Set tick labels
    if graph_settings['show_x_tick_labels'] is False:
        ax.set_xticklabels([])
    if graph_settings['show_y_tick_labels'] is False:
        ax.set_yticklabels([])


    # Add title, legend and labels
    #title = graph_settings['file_label']
    if graph_settings['show_title'] is True:
        plt.title(title,fontdict=graph_settings['title_font'])
        top_margin=0.9
    if graph_settings['show_ylabel'] is True:
        plt.ylabel(graph_settings['y_label'], fontdict=graph_settings['label_font'])
        left_margin=0.1
    if graph_settings['show_xlabel'] is True:
        plt.xlabel(graph_settings['x_label'], fontdict=graph_settings['label_font'])
        bottom_margin=0.11
    if graph_settings['show_legend'] is True:
        plt.legend(fontsize=graph_settings['legend_font_size'],frameon=graph_settings['show_legend_frame'],ncol=2,loc='best')

    #plt.subplots_adjust(left=left_margin,right=right_margin,bottom=bottom_margin,top=top_margin)
    plt.tight_layout()

    return


def save_to_file(input_settings, graph_settings):

    # This function saves the graphs to image files

    if graph_settings['export_svg'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.svg', format='svg')
    if graph_settings['export_png'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.png', format='png')

    return


if __name__ == "__main__":
    sys.exit(main())

